/*******************************************************************************
 *
 * Motorola Inc. 2003 All rights reserved 
 * 
 ******************************************************************************* 
 * 
 * $File Name        : sci.h$
 * $Date             : Oct-20-2003$
 * $Version          : 4.0.4.0$ 
 * $Last Modified By : r30323$
 * 
 * Description       : Bootloader Software SCI API wrapper
 * 
 ******************************************************************************/ 


/*

The most of HC08 MCUs can be programmed with so-called Bootloader application.
For details look for AN2295 Application Note on Motorola Website.

Since some HC08s has no hardware SCI available, the Bootloader emulates it using software.
User application now can re-use this software SCI algorithms to communicate over the
same pins.

The API consists of one table. The table *address* (pointer) is located
at unused interrupt vector location that is specific for each SCI-less HC08.

SCIAPI points to --> +-------------------------------+
                     | SCI init routine address      |
                     +-------------------------------+
                     | SCI RX routine address        |
                     +-------------------------------+
                     | SCI TX routine address        |
                     +-------------------------------+
                     | ONEBIT (run-time) address     |
                     +-------------------------------+

The table contains four entries:

* SCI init routine address        
  - this routine must be called before any SCI routine usage.
  - it will initialize DDR for TX pin.
  - it will initialize DDR & pull-up (if applicable) for RX pin.
  - it will also initialize timing values in RAM for SCI routines for 9600Bd rate

* SCI RX routine address
  - the receive routine, the value is returned in register A

* SCI TX routine address
  - the transmit routine, the value to send expected in register A

* ONEBIT (run-time) address
  - this variable contains the value that is proportional to the datarate. Upon exit
    from SCI init routine, it is filled with value for 9600Bd. If you wish to use e.g.
    half data rate, read this value and mutliply it by two (see example below).

There are just a few restrictions:

* User application may NOT use the three RAM locations specific for each HC08 (see below).
  Just tweak .prm file for the linker won't occupy these locations.
  
* Any interrupt source may corrupt the SCI communication so it's strongly adviced
  to disable interrupts during SCI communications.
  
* The SCI routines will use the timer. Only modulo counter (TMODH:TMODL) and 
  Timer Status & Control (TSC) registers are modified.
  the timer will be stopped and TOF flag cleared on exit from both SCI routines.



SCIAPI is stored at following addreses:

MC68HC908QT/QY  0xFFF8:0xFFF9
MC68HC908JK/JL  0xFFF8:0xFFF9

Reserved RAM locations are:

MC68HC908QT/QY  0x0080:0x0082 - total 3 bytes
MC68HC908JK/JL  0x0080:0x0082 - total 3 bytes


API usage - is simple :)
------------------------

#include "sci.h"            // just include this file ;)

unsigned char chr;
unsigned char scidelay;

void main(void)
{
    /-* MCU INIT HERE *-/
    
    SCIAPIInit();           // initialize SCI API
    
    scidelay = SCIAPISpeed;
    
    /-* more stuff *-/
    
    chr = 'A';
    SCIAPIWrite(chr);       // will send 'A'
    
    chr = SCIAPIRead();     // will read char from SCI


    SCIAPISpeed = 2 * scidelay;       // tweak communication speed for 4800Bd

    chr = 'B';
    SCIAPIWrite(chr);       // will send 'B' @ 4800 datarate
    
    chr = SCIAPIRead();     // will read char from SCI

}


*/

#ifndef _BOOTLOADERSCIAPI_
#define _BOOTLOADERSCIAPI_ 1

typedef unsigned char (*PTR2FUNRD)(void); 
typedef void (*PTR2FUNWR)(unsigned char); 
typedef void (*PTR2FUN)(void); 

#define SCIAPIADDR 0xfff8 /* QT/QY/JK/JL families */

#define SCIAPI    (*((unsigned int *)(SCIAPIADDR)))

#define SCIAPIInit ((PTR2FUN)( *(((unsigned int *)SCIAPI)+0) ))
#define SCIAPIRead ((PTR2FUNRD)( *(((unsigned int *)SCIAPI)+1) ))
#define SCIAPIWrite ((PTR2FUNWR)( *(((unsigned int *)SCIAPI)+2) ))
#define SCIAPISpeed (*(unsigned int *)( *(((unsigned int *)SCIAPI)+3) ))

#endif
